@@ -57,6 +57,18 @@ new_bitarray(Py_ssize_t nbits, PyObject *endian)
     return res;
 }
 
+/* Starting from word index `i`, count the remaining population in bitarray
+   buffer.  Equivalent to:  a[64 * i:].count()  */
+static Py_ssize_t
+count_from_word(bitarrayobject *a, Py_ssize_t i)
+{
+    assert(i >= 0);
+    if (64 * i >= a->nbits)
+        return 0;
+
+    return popcnt_words(WBUFF(a) + i, a->nbits / 64 - i) + popcnt_64(zlw(a));
+}
+
 /* -------------------------------- zeros ------------------------------ */
 
 static PyObject *
@@ -89,9 +101,9 @@ endianness, which may be 'big', 'little'.");
 /* ------------------------------- count_n ----------------------------- */
 
 /* Return the smallest index i for which a.count(vi, 0, i) == n.
-   When n exceeds the total count, the result is a always negative
-   number (the negative of the total count + 1, which is useful
-   for displaying the error message). */
+   When n exceeds the total count, the result is a negative
+   number; the negative of the total count + 1, which is useful
+   for displaying the error message. */
 static Py_ssize_t
 count_n_core(bitarrayobject *a, Py_ssize_t n, int vi)
 {
@@ -102,8 +114,6 @@ count_n_core(bitarrayobject *a, Py_ssize_t n, int vi)
     Py_ssize_t m;             /* popcount in each block */
 
     assert(0 <= n && n <= nbits);
-    if (n == 0)
-        return 0;
 
     /* by counting big blocks we save comparisons and updates */
 #define BLOCK_BITS  4096      /* block size: 4096 bits = 64 words */
@@ -129,12 +139,14 @@ count_n_core(bitarrayobject *a, Py_ssize_t n, int vi)
     }
 
     while (i < nbits && t < n) {
-        t += getbit(a, i++) == vi;
+        t += getbit(a, i) == vi;
+        i++;
     }
 
-    if (t < n)  /* n exceeds total count */
+    if (t < n) {  /* n exceeds total count */
+        assert((vi ? t : nbits - t) == count_from_word(a, 0));
         return -(t + 1);
-
+    }
     return i;
 }
 
@@ -1028,22 +1040,6 @@ byte_length(Py_ssize_t i)
     return n;
 }
 
-/* Starting from word index `i`, count the remaining population in bitarray
-   buffer.  Equivalent to:  a[64 * i:].count()  */
-static Py_ssize_t
-count_from_word(bitarrayobject *a, Py_ssize_t i)
-{
-    const Py_ssize_t nbits = a->nbits;
-    Py_ssize_t cnt = 0;
-
-    if (64 * i >= nbits)
-        return 0;
-    cnt += popcnt_words(WBUFF(a) + i, nbits / 64 - i);
-    if (nbits % 64)
-        cnt += popcnt_64(zlw(a));
-    return cnt;
-}
-
 /* ---------------------- sparse compressed bitarray -------------------
  *
  * see also: doc/sparse_compression.rst
@@ -1063,7 +1059,7 @@ count_from_word(bitarrayobject *a, Py_ssize_t i)
    is constructed around this. */
 #define SEGSIZE  32
 
-/* number of 256 bit segments given nbits */
+/* number of segments for given number of bits */
 #define NSEG(nbits)  (((nbits) + 8 * SEGSIZE - 1) / (8 * SEGSIZE))
 
 /* Calculate an array with the running totals (rts) for 256 bit segments.
@@ -1109,9 +1105,9 @@ count_from_word(bitarrayobject *a, Py_ssize_t i)
 static Py_ssize_t *
 sc_calc_rts(bitarrayobject *a)
 {
-    const Py_ssize_t nbits = a->nbits;
-    const Py_ssize_t n_seg = NSEG(nbits);     /* number of segments */
-    const Py_ssize_t c_seg = nbits / (8 * SEGSIZE);  /* complete segments */
+    /* total number of segments - number of complete segments */
+    const Py_ssize_t n_seg = NSEG(a->nbits);
+    const Py_ssize_t c_seg = a->nbits / (8 * SEGSIZE);
     char zeros[SEGSIZE];                      /* segment with only zeros */
     Py_ssize_t cnt = 0;                       /* current count */
     char *buff;                               /* buffer in current segment */
@@ -1134,7 +1130,7 @@ sc_calc_rts(bitarrayobject *a)
     res[c_seg] = cnt;
 
     if (n_seg > c_seg) {           /* we have a final partial segment */
-        cnt += count_from_word(a, (SEGSIZE / 8) * c_seg);
+        cnt += count_from_word(a, c_seg * SEGSIZE / 8);
         res[n_seg] = cnt;
     }
     return res;
