@@ -552,9 +552,10 @@ class TestsCount_N(unittest.TestCase, Util):
             tc = a.count(v)      # total count
             i = count_n(a, tc, v)
             self.check_result(a, tc, i, v)
+            n = tc + 1
             self.assertRaisesMessage(ValueError, "n = %d exceeds total count "
-                                     "(a.count(%d) = %d)" % (tc + 1, v, tc),
-                                     count_n, a, tc + 1, v)
+                                     "(a.count(%d) = %d)" % (n, v, tc),
+                                     count_n, a, n, v)
             for _ in range(20):
                 n = randint(0, tc)
                 i = count_n(a, n, v)
@@ -566,6 +567,8 @@ class TestsCount_N(unittest.TestCase, Util):
                 n = a.count(v) // 2
                 i = count_n(a, n, v)
                 self.check_result(a, n, i, v)
+                # n = 0 -> count_n always 0
+                self.assertEqual(count_n(a, 0, v), 0)
 
 tests.append(TestsCount_N)
 
